@@ -65,20 +65,25 @@ struct Keygroups<'a> {
 
 fn add_keybinds(help: &ModeInfo) -> Keygroups {
     let normal_keymap = help.get_mode_keybinds();
-    let new_pane_keys = action_key(&normal_keymap, &[Action::NewPane(None)]);
+    let new_pane_keys = action_key(&normal_keymap, &[Action::NewPane(None, None)]);
     let new_pane = if new_pane_keys.is_empty() {
         vec![Style::new().bold().paint("UNBOUND")]
     } else {
         style_key_with_modifier(&new_pane_keys, &help.style.colors)
     };
 
-    let resize_keys = action_key_group(
+    let mut resize_keys = action_key_group(
         &normal_keymap,
         &[
             &[Action::Resize(ResizeDirection::Increase)],
             &[Action::Resize(ResizeDirection::Decrease)],
         ],
     );
+    if resize_keys.contains(&Key::Alt(CharOrArrow::Char('=')))
+        && resize_keys.contains(&Key::Alt(CharOrArrow::Char('+')))
+    {
+        resize_keys.retain(|k| k != &Key::Alt(CharOrArrow::Char('=')));
+    }
     let resize = if resize_keys.is_empty() {
         vec![Style::new().bold().paint("UNBOUND")]
     } else {
