@@ -6,13 +6,18 @@ use crate::sessions::{
     session_exists, ActiveSession, SessionNameMatch,
 };
 use dialoguer::Confirm;
-use miette::Result;
+use miette::{Report, Result};
 use std::path::PathBuf;
 use std::process;
+use zellij_client::old_config_converter::{
+    config_yaml_to_config_kdl, convert_old_yaml_files, layout_yaml_to_layout_kdl,
+};
 use zellij_client::start_client as start_client_impl;
 use zellij_client::{os_input_output::get_client_os_input, ClientInfo};
 use zellij_server::os_input_output::get_server_os_input;
 use zellij_server::start_server as start_server_impl;
+use zellij_utils::input::actions::Action;
+use zellij_utils::input::config::ConfigError;
 use zellij_utils::input::options::Options;
 use zellij_utils::nix;
 use zellij_utils::{
@@ -21,10 +26,7 @@ use zellij_utils::{
     setup::{get_default_data_dir, Setup},
 };
 
-#[cfg(feature = "unstable")]
-use miette::IntoDiagnostic;
-#[cfg(feature = "unstable")]
-use zellij_utils::input::actions::ActionsFromYaml;
+use std::{fs::File, io::prelude::*};
 
 pub(crate) use crate::sessions::list_sessions;
 
@@ -84,7 +86,9 @@ fn get_os_input<OsInputOutput>(
     }
 }
 
-pub(crate) fn start_server(path: PathBuf) {
+pub(crate) fn start_server(path: PathBuf, debug: bool) {
+    // Set instance-wide debug mode
+    zellij_utils::consts::DEBUG_MODE.set(debug).unwrap();
     let os_input = get_os_input(get_server_os_input);
     start_server_impl(Box::new(os_input), path);
 }
@@ -118,69 +122,133 @@ fn find_indexed_session(
     }
 }
 
-/// Send a vec of `[Action]` to a currently running session.
-#[cfg(feature = "unstable")]
-pub(crate) fn send_action_to_session(opts: zellij_utils::cli::CliArgs) {
+pub(crate) fn send_action_to_session(
+    cli_action: zellij_utils::cli::CliAction,
+    requested_session_name: Option<String>,
+) {
     match get_active_session() {
         ActiveSession::None => {
             eprintln!("There is no active session!");
             std::process::exit(1);
         },
         ActiveSession::One(session_name) => {
-            attach_with_fake_client(opts, &session_name);
+            if let Some(requested_session_name) = requested_session_name {
+                if requested_session_name != session_name {
+                    eprintln!(
+                        "Session '{}' not found. The following sessions are active:",
+                        requested_session_name
+                    );
+                    eprintln!("{}", session_name);
+                    std::process::exit(1);
+                }
+            }
+            attach_with_cli_client(cli_action, &session_name);
         },
         ActiveSession::Many => {
-            if let Some(session_name) = opts.session.clone() {
-                attach_with_fake_client(opts, &session_name);
+            let existing_sessions = get_sessions().unwrap();
+            if let Some(session_name) = requested_session_name {
+                if existing_sessions.contains(&session_name) {
+                    attach_with_cli_client(cli_action, &session_name);
+                } else {
+                    eprintln!(
+                        "Session '{}' not found. The following sessions are active:",
+                        session_name
+                    );
+                    print_sessions(existing_sessions);
+                    std::process::exit(1);
+                }
             } else if let Ok(session_name) = envs::get_session_name() {
-                attach_with_fake_client(opts, &session_name);
+                attach_with_cli_client(cli_action, &session_name);
             } else {
-                println!("Please specify the session name to send actions to. The following sessions are active:");
-                print_sessions(get_sessions().unwrap());
+                eprintln!("Please specify the session name to send actions to. The following sessions are active:");
+                print_sessions(existing_sessions);
                 std::process::exit(1);
             }
         },
     };
 }
+pub(crate) fn convert_old_config_file(old_config_file: PathBuf) {
+    match File::open(&old_config_file) {
+        Ok(mut handle) => {
+            let mut raw_config_file = String::new();
+            let _ = handle.read_to_string(&mut raw_config_file);
+            match config_yaml_to_config_kdl(&raw_config_file, false) {
+                Ok(kdl_config) => {
+                    println!("{}", kdl_config);
+                    process::exit(0);
+                },
+                Err(e) => {
+                    eprintln!("Failed to convert config: {}", e);
+                    process::exit(1);
+                },
+            }
+        },
+        Err(e) => {
+            eprintln!("Failed to open file: {}", e);
+            process::exit(1);
+        },
+    }
+}
 
-#[cfg(feature = "unstable")]
-fn attach_with_fake_client(opts: zellij_utils::cli::CliArgs, name: &str) {
-    if let Some(zellij_utils::cli::Command::Sessions(zellij_utils::cli::Sessions::Action {
-        action: Some(action),
-    })) = opts.command.clone()
-    {
-        let action = format!("[{}]", action);
-        match zellij_utils::serde_yaml::from_str::<ActionsFromYaml>(&action).into_diagnostic() {
-            Ok(parsed) => {
-                let (config, _, config_options) = match Setup::from_options(&opts) {
-                    Ok(results) => results,
-                    Err(e) => {
-                        eprintln!("{}", e);
-                        process::exit(1);
-                    },
-                };
-                let os_input = get_os_input(zellij_client::os_input_output::get_client_os_input);
+pub(crate) fn convert_old_layout_file(old_layout_file: PathBuf) {
+    match File::open(&old_layout_file) {
+        Ok(mut handle) => {
+            let mut raw_layout_file = String::new();
+            let _ = handle.read_to_string(&mut raw_layout_file);
+            match layout_yaml_to_layout_kdl(&raw_layout_file) {
+                Ok(kdl_layout) => {
+                    println!("{}", kdl_layout);
+                    process::exit(0);
+                },
+                Err(e) => {
+                    eprintln!("Failed to convert layout: {}", e);
+                    process::exit(1);
+                },
+            }
+        },
+        Err(e) => {
+            eprintln!("Failed to open file: {}", e);
+            process::exit(1);
+        },
+    }
+}
 
-                let actions = parsed.actions().to_vec();
-                log::debug!("Starting fake Zellij client!");
-                zellij_client::fake_client::start_fake_client(
-                    Box::new(os_input),
-                    opts,
-                    *Box::new(config),
-                    config_options,
-                    ClientInfo::New(name.to_string()),
-                    None,
-                    actions,
-                );
-                log::debug!("Quitting fake client now.");
-                std::process::exit(0);
-            },
-            Err(e) => {
-                eprintln!("{:?}", e);
-                std::process::exit(1);
-            },
-        };
-    };
+pub(crate) fn convert_old_theme_file(old_theme_file: PathBuf) {
+    match File::open(&old_theme_file) {
+        Ok(mut handle) => {
+            let mut raw_config_file = String::new();
+            let _ = handle.read_to_string(&mut raw_config_file);
+            match config_yaml_to_config_kdl(&raw_config_file, true) {
+                Ok(kdl_config) => {
+                    println!("{}", kdl_config);
+                    process::exit(0);
+                },
+                Err(e) => {
+                    eprintln!("Failed to convert config: {}", e);
+                    process::exit(1);
+                },
+            }
+        },
+        Err(e) => {
+            eprintln!("Failed to open file: {}", e);
+            process::exit(1);
+        },
+    }
+}
+
+fn attach_with_cli_client(cli_action: zellij_utils::cli::CliAction, session_name: &str) {
+    let os_input = get_os_input(zellij_client::os_input_output::get_client_os_input);
+    match Action::actions_from_cli(cli_action) {
+        Ok(actions) => {
+            zellij_client::cli_client::start_cli_client(Box::new(os_input), session_name, actions);
+            std::process::exit(0);
+        },
+        Err(e) => {
+            eprintln!("{}", e);
+            log::error!("Error sending action: {}", e);
+            std::process::exit(2);
+        },
+    }
 }
 
 fn attach_with_session_index(config_options: Options, index: usize, create: bool) -> ClientInfo {
@@ -209,10 +277,10 @@ fn attach_with_session_name(
 ) -> ClientInfo {
     match &session_name {
         Some(session) if create => {
-            if !session_exists(session).unwrap() {
-                ClientInfo::New(session_name.unwrap())
-            } else {
+            if session_exists(session).unwrap() {
                 ClientInfo::Attach(session_name.unwrap(), config_options)
+            } else {
+                ClientInfo::New(session_name.unwrap())
             }
         },
         Some(prefix) => match match_session_name(prefix).unwrap() {
@@ -249,10 +317,17 @@ fn attach_with_session_name(
 }
 
 pub(crate) fn start_client(opts: CliArgs) {
-    let (config, layout, config_options) = match Setup::from_options(&opts) {
+    // look for old YAML config/layout/theme files and convert them to KDL
+    convert_old_yaml_files(&opts);
+    let (config, layout, config_options) = match Setup::from_cli_args(&opts) {
         Ok(results) => results,
         Err(e) => {
-            eprintln!("{}", e);
+            if let ConfigError::KdlError(error) = e {
+                let report: Report = error.into();
+                eprintln!("{:?}", report);
+            } else {
+                eprintln!("{}", e);
+            }
             process::exit(1);
         },
     };
@@ -278,14 +353,14 @@ pub(crate) fn start_client(opts: CliArgs) {
 
         if let Ok(val) = std::env::var(envs::SESSION_NAME_ENV_KEY) {
             if val == *client.get_session_name() {
-                eprintln!("You are trying to attach to the current session(\"{}\"). Zellij does not support nesting a session in itself.", val);
+                eprintln!("You are trying to attach to the current session (\"{}\"). Zellij does not support nesting a session in itself.", val);
                 process::exit(1);
             }
         }
 
         let attach_layout = match client {
             ClientInfo::Attach(_, _) => None,
-            ClientInfo::New(_) => layout,
+            ClientInfo::New(_) => Some(layout),
         };
 
         if create {
@@ -314,23 +389,34 @@ pub(crate) fn start_client(opts: CliArgs) {
                 config,
                 config_options,
                 ClientInfo::New(session_name),
-                layout,
+                Some(layout),
             );
         } else {
-            if let Some(layout_some) = layout.clone() {
-                if let Some(session_name) = layout_some.session.name {
-                    if layout_some.session.attach.unwrap() {
+            if let Some(session_name) = config_options.session_name.as_ref() {
+                if let Ok(val) = envs::get_session_name() {
+                    // This prevents the same type of recursion as above, only that here we
+                    // don't get the command to "attach", but to start a new session instead.
+                    // This occurs for example when declaring the session name inside a layout
+                    // file and then, from within this session, trying to open a new zellij
+                    // session with the same layout. This causes an infinite recursion in the
+                    // `zellij_server::terminal_bytes::listen` task, flooding the server and
+                    // clients with infinite `Render` requests.
+                    if *session_name == val {
+                        eprintln!("You are trying to attach to the current session (\"{}\"). Zellij does not support nesting a session in itself.", session_name);
+                        process::exit(1);
+                    }
+                }
+                match config_options.attach_to_session {
+                    Some(true) => {
                         let client = attach_with_session_name(
-                            Some(session_name),
+                            Some(session_name.clone()),
                             config_options.clone(),
                             true,
                         );
-
                         let attach_layout = match client {
                             ClientInfo::Attach(_, _) => None,
-                            ClientInfo::New(_) => layout,
+                            ClientInfo::New(_) => Some(layout),
                         };
-
                         start_client_impl(
                             Box::new(os_input),
                             opts,
@@ -339,20 +425,23 @@ pub(crate) fn start_client(opts: CliArgs) {
                             client,
                             attach_layout,
                         );
-                    } else {
+                    },
+                    _ => {
                         start_client_plan(session_name.clone());
                         start_client_impl(
                             Box::new(os_input),
                             opts,
                             config,
-                            config_options,
-                            ClientInfo::New(session_name),
-                            layout,
+                            config_options.clone(),
+                            ClientInfo::New(session_name.clone()),
+                            Some(layout),
                         );
-                    }
-
-                    process::exit(0);
+                    },
                 }
+                // after we detach, this happens and so we need to exit before the rest of the
+                // function happens
+                // TODO: offload this to a different function
+                process::exit(0);
             }
 
             let session_name = names::Generator::default().next().unwrap();
@@ -363,7 +452,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                 config,
                 config_options,
                 ClientInfo::New(session_name),
-                layout,
+                Some(layout),
             );
         }
     }
