@@ -13,6 +13,7 @@ use super::remote_runner::{RemoteRunner, RemoteTerminal, Step};
 pub const QUIT: [u8; 1] = [17]; // ctrl-q
 pub const ESC: [u8; 1] = [27];
 pub const ENTER: [u8; 1] = [10]; // char '\n'
+pub const SPACE: [u8; 1] = [32];
 pub const LOCK_MODE: [u8; 1] = [7]; // ctrl-g
 
 pub const MOVE_FOCUS_LEFT_IN_NORMAL_MODE: [u8; 2] = [27, 104]; // alt-h
@@ -458,9 +459,8 @@ pub fn close_tab() {
             name: "Wait for tab to close",
             instruction: |mut remote_terminal: RemoteTerminal| -> bool {
                 let mut step_is_complete = false;
-                if remote_terminal.cursor_position_is(3, 2)
+                if remote_terminal.snapshot_contains("Tab #1")
                     && !remote_terminal.snapshot_contains("Tab #2")
-                    && remote_terminal.tip_appears()
                 {
                     // cursor is in the first tab again
                     step_is_complete = true;
@@ -475,7 +475,8 @@ pub fn close_tab() {
             break last_snapshot;
         }
     };
-    assert_snapshot!(last_snapshot);
+    assert!(last_snapshot.contains("Tab #1"));
+    assert!(!last_snapshot.contains("Tab #2"));
 }
 
 #[test]
@@ -948,41 +949,6 @@ pub fn detach_and_attach_session() {
     assert_snapshot!(last_snapshot);
 }
 
-#[test]
-#[ignore]
-pub fn accepts_basic_layout() {
-    let fake_win_size = Size {
-        cols: 120,
-        rows: 24,
-    };
-    let layout_file_name = "three-panes-with-nesting.yaml";
-    let mut test_attempts = 10;
-    let last_snapshot = loop {
-        RemoteRunner::kill_running_sessions(fake_win_size);
-        let mut runner = RemoteRunner::new_with_layout(fake_win_size, layout_file_name);
-        runner.run_all_steps();
-        let last_snapshot = runner.take_snapshot_after(Step {
-            name: "Wait for app to load",
-            instruction: |remote_terminal: RemoteTerminal| -> bool {
-                let mut step_is_complete = false;
-                if remote_terminal.cursor_position_is(3, 1)
-                    && remote_terminal.snapshot_contains("$ █                   ││$")
-                    && remote_terminal.snapshot_contains("$                                                                                                                     ") {
-                    step_is_complete = true;
-                }
-                step_is_complete
-            },
-        });
-        if runner.test_timed_out && test_attempts > 0 {
-            test_attempts -= 1;
-            continue;
-        } else {
-            break last_snapshot;
-        }
-    };
-    assert_snapshot!(last_snapshot);
-}
-
 #[test]
 #[ignore]
 pub fn status_bar_loads_custom_keybindings() {
@@ -990,7 +956,7 @@ pub fn status_bar_loads_custom_keybindings() {
         cols: 120,
         rows: 24,
     };
-    let config_file_name = "changed_keys.yaml";
+    let config_file_name = "changed_keys.kdl";
     let mut test_attempts = 10;
     let last_snapshot = loop {
         RemoteRunner::kill_running_sessions(fake_win_size);
@@ -1718,43 +1684,6 @@ pub fn toggle_floating_panes() {
     assert_snapshot!(last_snapshot);
 }
 
-#[test]
-#[ignore]
-pub fn focus_tab_with_layout() {
-    let fake_win_size = Size {
-        cols: 120,
-        rows: 24,
-    };
-    let layout_file_name = "focus-tab-layout.yaml";
-    let mut test_attempts = 10;
-    let last_snapshot = loop {
-        RemoteRunner::kill_running_sessions(fake_win_size);
-        let mut runner = RemoteRunner::new_with_layout(fake_win_size, layout_file_name);
-        runner.run_all_steps();
-        let last_snapshot = runner.take_snapshot_after(Step {
-            name: "Wait for app to load",
-            instruction: |remote_terminal: RemoteTerminal| -> bool {
-                let mut step_is_complete = false;
-                if remote_terminal.status_bar_appears()
-                    && remote_terminal.tip_appears()
-                    && remote_terminal.snapshot_contains("Tab #9")
-                    && remote_terminal.cursor_position_is(63, 2)
-                {
-                    step_is_complete = true;
-                }
-                step_is_complete
-            },
-        });
-        if runner.test_timed_out && test_attempts > 0 {
-            test_attempts -= 1;
-            continue;
-        } else {
-            break last_snapshot;
-        }
-    };
-    assert_snapshot!(last_snapshot);
-}
-
 #[test]
 #[ignore]
 pub fn tmux_mode() {
@@ -1948,3 +1877,93 @@ pub fn undo_rename_pane() {
     };
     assert_snapshot!(last_snapshot);
 }
+
+#[test]
+#[ignore]
+pub fn send_command_through_the_cli() {
+    // here we test the following flow:
+    // - send a command through the cli to run a bash script in a temporary folder
+    // - have it open a "command pane" that can be re-run with Enter
+    // - press Enter in the command pane to re-run the script
+    //
+    // the script appends the word "foo" to a temporary file and then `cat`s that file,
+    // so when we press "Enter", it will run again and we'll see two "foo"s one after the other,
+    // that's how we know the whole flow is working
+    let fake_win_size = Size {
+        cols: 120,
+        rows: 24,
+    };
+    let mut test_attempts = 10;
+    let last_snapshot = loop {
+        RemoteRunner::kill_running_sessions(fake_win_size);
+        let mut runner = RemoteRunner::new(fake_win_size)
+            .add_step(Step {
+                name: "Run command through the cli",
+                instruction: |mut remote_terminal: RemoteTerminal| -> bool {
+                    let mut step_is_complete = false;
+                    if remote_terminal.status_bar_appears()
+                        && remote_terminal.cursor_position_is(3, 2)
+                    {
+                        let fixture_folder = remote_terminal.path_to_fixture_folder();
+                        remote_terminal.send_command_through_the_cli(&format!(
+                            "{}/append-echo-script.sh",
+                            fixture_folder
+                        ));
+                        step_is_complete = true;
+                    }
+                    step_is_complete
+                },
+            })
+            .add_step(Step {
+                name: "Wait for command to run",
+                instruction: |mut remote_terminal: RemoteTerminal| -> bool {
+                    let mut step_is_complete = false;
+                    if remote_terminal.snapshot_contains("<Ctrl-c>")
+                        && remote_terminal.cursor_position_is(61, 3)
+                    {
+                        remote_terminal.send_key(&SPACE); // re-run script - here we use SPACE
+                                                          // instead of the default ENTER because
+                                                          // sending ENTER over SSH can be a little
+                                                          // problematic (read: I couldn't get it
+                                                          // to pass consistently)
+                        step_is_complete = true
+                    }
+                    step_is_complete
+                },
+            })
+            .add_step(Step {
+                name: "Wait for script to run again",
+                instruction: |mut remote_terminal: RemoteTerminal| -> bool {
+                    let mut step_is_complete = false;
+                    if remote_terminal.snapshot_contains("<Ctrl-c>")
+                        && remote_terminal.cursor_position_is(61, 4)
+                    {
+                        step_is_complete = true
+                    }
+                    step_is_complete
+                },
+            });
+        runner.run_all_steps();
+
+        let last_snapshot = runner.take_snapshot_after(Step {
+            name: "Wait for script to run twice",
+            instruction: |remote_terminal: RemoteTerminal| -> bool {
+                let mut step_is_complete = false;
+                if remote_terminal.snapshot_contains("foo")
+                    && remote_terminal.cursor_position_is(61, 4)
+                {
+                    step_is_complete = true
+                }
+                step_is_complete
+            },
+        });
+
+        if runner.test_timed_out && test_attempts > 0 {
+            test_attempts -= 1;
+            continue;
+        } else {
+            break last_snapshot;
+        }
+    };
+    assert_snapshot!(last_snapshot);
+}
