@@ -14,16 +14,17 @@ use std::rc::Rc;
 use std::time::Instant;
 use zellij_utils::{
     data::{ModeInfo, Style},
-    input::layout::Direction,
+    input::command::RunCommand,
+    input::layout::SplitDirection,
     pane_size::{Offset, PaneGeom, Size, SizeInPixels, Viewport},
 };
 
 macro_rules! resize_pty {
     ($pane:expr, $os_input:expr) => {
         if let PaneId::Terminal(ref pid) = $pane.pid() {
-            // FIXME: This `set_terminal_size_using_fd` call would be best in
+            // FIXME: This `set_terminal_size_using_terminal_id` call would be best in
             // `TerminalPane::reflow_lines`
-            $os_input.set_terminal_size_using_fd(
+            $os_input.set_terminal_size_using_terminal_id(
                 *pid,
                 $pane.get_content_columns() as u16,
                 $pane.get_content_rows() as u16,
@@ -211,7 +212,7 @@ impl TiledPanes {
     pub fn pane_ids(&self) -> impl Iterator<Item = &PaneId> {
         self.panes.keys()
     }
-    pub fn relayout(&mut self, direction: Direction) {
+    pub fn relayout(&mut self, direction: SplitDirection) {
         let mut pane_grid = TiledPaneGrid::new(
             &mut self.panes,
             &self.panes_to_hide,
@@ -219,10 +220,12 @@ impl TiledPanes {
             *self.viewport.borrow(),
         );
         let result = match direction {
-            Direction::Horizontal => {
+            SplitDirection::Horizontal => {
                 pane_grid.layout(direction, (*self.display_area.borrow()).cols)
             },
-            Direction::Vertical => pane_grid.layout(direction, (*self.display_area.borrow()).rows),
+            SplitDirection::Vertical => {
+                pane_grid.layout(direction, (*self.display_area.borrow()).rows)
+            },
         };
         if let Err(e) = &result {
             log::error!("{:?} relayout of the tab failed: {}", direction, e);
@@ -269,7 +272,7 @@ impl TiledPanes {
                 if full_pane_size.rows.as_usize() < MIN_TERMINAL_HEIGHT * 2 {
                     return false;
                 } else {
-                    return split(Direction::Horizontal, &full_pane_size).is_some();
+                    return split(SplitDirection::Horizontal, &full_pane_size).is_some();
                 }
             }
         }
@@ -282,7 +285,7 @@ impl TiledPanes {
                 if full_pane_size.cols.as_usize() < MIN_TERMINAL_WIDTH * 2 {
                     return false;
                 }
-                return split(Direction::Vertical, &full_pane_size).is_some();
+                return split(SplitDirection::Vertical, &full_pane_size).is_some();
             }
         }
         false
@@ -296,11 +299,13 @@ impl TiledPanes {
         let active_pane_id = &self.active_panes.get(&client_id).unwrap();
         let active_pane = self.panes.get_mut(active_pane_id).unwrap();
         let full_pane_size = active_pane.position_and_size();
-        if let Some((top_winsize, bottom_winsize)) = split(Direction::Horizontal, &full_pane_size) {
+        if let Some((top_winsize, bottom_winsize)) =
+            split(SplitDirection::Horizontal, &full_pane_size)
+        {
             active_pane.set_geom(top_winsize);
             new_pane.set_geom(bottom_winsize);
             self.panes.insert(pid, new_pane);
-            self.relayout(Direction::Vertical);
+            self.relayout(SplitDirection::Vertical);
         }
     }
     pub fn split_pane_vertically(
@@ -312,11 +317,13 @@ impl TiledPanes {
         let active_pane_id = &self.active_panes.get(&client_id).unwrap();
         let active_pane = self.panes.get_mut(active_pane_id).unwrap();
         let full_pane_size = active_pane.position_and_size();
-        if let Some((left_winsize, right_winsize)) = split(Direction::Vertical, &full_pane_size) {
+        if let Some((left_winsize, right_winsize)) =
+            split(SplitDirection::Vertical, &full_pane_size)
+        {
             active_pane.set_geom(left_winsize);
             new_pane.set_geom(right_winsize);
             self.panes.insert(pid, new_pane);
-            self.relayout(Direction::Horizontal);
+            self.relayout(SplitDirection::Horizontal);
         }
     }
     pub fn focus_pane(&mut self, pane_id: PaneId, client_id: ClientId) {
@@ -455,15 +462,18 @@ impl TiledPanes {
                 *display_area,
                 *viewport,
             );
-            if pane_grid.layout(Direction::Horizontal, cols).is_ok() {
-                let column_difference = cols as isize - display_area.cols as isize;
-                // FIXME: Should the viewport be an Offset?
-                viewport.cols = (viewport.cols as isize + column_difference) as usize;
-                display_area.cols = cols;
-            } else {
-                log::error!("Failed to horizontally resize the tab!!!");
-            }
-            if pane_grid.layout(Direction::Vertical, rows).is_ok() {
+            match pane_grid.layout(SplitDirection::Horizontal, cols) {
+                Ok(_) => {
+                    let column_difference = cols as isize - display_area.cols as isize;
+                    // FIXME: Should the viewport be an Offset?
+                    viewport.cols = (viewport.cols as isize + column_difference) as usize;
+                    display_area.cols = cols;
+                },
+                Err(e) => {
+                    log::error!("Failed to horizontally resize the tab: {:?}", e);
+                },
+            };
+            if pane_grid.layout(SplitDirection::Vertical, rows).is_ok() {
                 let row_difference = rows as isize - display_area.rows as isize;
                 viewport.rows = (viewport.rows as isize + row_difference) as usize;
                 display_area.rows = rows;
@@ -972,9 +982,7 @@ impl TiledPanes {
             // successfully filled space over pane
             let closed_pane = self.panes.remove(&pane_id);
             self.move_clients_out_of_pane(pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api);
-            }
+            self.set_pane_frames(self.draw_pane_frames); // recalculate pane frames and update size
             closed_pane
         } else {
             self.panes.remove(&pane_id);
@@ -984,6 +992,16 @@ impl TiledPanes {
             None
         }
     }
+    pub fn hold_pane(
+        &mut self,
+        pane_id: PaneId,
+        exit_status: Option<i32>,
+        run_command: RunCommand,
+    ) {
+        self.panes
+            .get_mut(&pane_id)
+            .map(|p| p.hold(exit_status, run_command));
+    }
     pub fn panes_to_hide_contains(&self, pane_id: PaneId) -> bool {
         self.panes_to_hide.contains(&pane_id)
     }
