@@ -17,6 +17,24 @@ pub struct RunCommand {
     pub args: Vec<String>,
     #[serde(default)]
     pub cwd: Option<PathBuf>,
+    #[serde(default)]
+    pub hold_on_close: bool,
+}
+
+impl std::fmt::Display for RunCommand {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut command: String = self
+            .command
+            .as_path()
+            .as_os_str()
+            .to_string_lossy()
+            .to_string();
+        for arg in &self.args {
+            command.push(' ');
+            command.push_str(arg);
+        }
+        write!(f, "{}", command)
+    }
 }
 
 /// Intermediate representation
@@ -30,6 +48,8 @@ pub struct RunCommandAction {
     pub cwd: Option<PathBuf>,
     #[serde(default)]
     pub direction: Option<Direction>,
+    #[serde(default)]
+    pub hold_on_close: bool,
 }
 
 impl From<RunCommandAction> for RunCommand {
@@ -38,6 +58,7 @@ impl From<RunCommandAction> for RunCommand {
             command: action.command,
             args: action.args,
             cwd: action.cwd,
+            hold_on_close: action.hold_on_close,
         }
     }
 }
